﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DEVSsharp;

namespace Ispit
{
    class Lopta : Atomic
    {
        List<PortValue> buffer;
        Vector pos, v0,v;
        double r, e_l;
        double g = 9.81;
        public OutputPort OP;
        public InputPort lvIP, lpIP;
        public InputPort kolizijaIP;

        public Lopta(string name, TimeUnit tu, double r, Vector pos, Vector v0, double e_l ) : base(name, tu)
        {
            this.r = r;
            this.pos = pos;
            this.v0 = v0;
            this.e_l = e_l;
            OP = AddOP("OP");
            lvIP = AddIP("lvIP");
            lpIP = AddIP("lpIP");
            kolizijaIP = AddIP("kolizijaIP");
            buffer = new List<PortValue>();
        }

        public override bool delta_x(PortValue x)
        {
            if ( x.port == lvIP )
            {
                v = x.value as Vector;
                return true;
            }
            else if ( x.port == lpIP )
            {
                v = new Vector(0.0, 0.0, 0.0);
                pos = x.value as Vector;
                return true;
            }
            else if ( x.port == kolizijaIP )
            {
                LoptaInfo l = x.value as LoptaInfo;
                pos.x += v.x * TimeElapsed;
                pos.y += v.y * TimeElapsed;
                pos.z += v.z * TimeElapsed;

                buffer.Add(new PortValue(OP, pos));

                v = l.v;

                return true;
            }

            return false;
        }

        public override void delta_y(ref PortValue y)
        {
            if ( buffer.Count > 0 )
            {
                y = buffer[0];
                buffer.RemoveAt(0);
                return;
            }
            pos.x += v.x * TimeElapsed;
            pos.y += v.y * TimeElapsed;
            pos.z += v.z * TimeElapsed;

            buffer.Add(new PortValue(OP, pos));

            v.z -= g * TimeElapsed;
        }

        public override void init()
        {
            v = v0;
            buffer.Add(new PortValue(OP, pos));
        }

        public override double tau()
        {
            if (buffer.Count > 0) return 0;
            return e_l / Math.Sqrt(v.x * v.x + v.y * v.y + v.z * v.z);
        }

        public override string Get_s()
        {
            return base.Get_s() + " " + v.x + " " + v.y + " " + v.z + "\n" + pos.x + " " + pos.y + " " + pos.z;
        }
    }
}