﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DEVSsharp;

namespace Ispit
{
    class Povrsina : Atomic
    {
        double a, b;
        Vector pos;
        public InputPort kolizijaIP;
        public OutputPort loptaOP;
        public OutputPort povrsinaOP;
        List<PortValue> buffer;
        public OutputPort zvukOP;
        double z0, z;
        double kz = 1;
        double ez = 0.1;
        double az = 1;

        public Povrsina(string name, TimeUnit tu, Vector pos, double a,double b) : base(name, tu)
        {
            this.pos = pos;
            this.a = a;
            this.b = b;
            kolizijaIP = AddIP("kolizijaIP");
            loptaOP = AddOP("loptaOP");
            zvukOP = AddOP("zvukOP");
            povrsinaOP = AddOP("povrsinaOP");
            buffer = new List<PortValue>();
        }

        public override bool delta_x(PortValue x)
        {
            if ( x.port == kolizijaIP )
            {
                LoptaInfo l = x.value as LoptaInfo;
                l.v.x *= 0.9;
                l.v.y *= 0.9;
                l.v.z *= 0.9;

                if ( pos.z == 0 ) // da li je pod
                {
                    l.v.z *= -1;
                    z0 = Math.Abs( l.v.z ) * kz;
                }
                else // da li je verikalna tabla
                {
                    l.v.x *= -1;
                    z0 = Math.Abs( l.v.x ) * kz;
                }

                z = z0;

                buffer.Add(new PortValue(loptaOP, l));
                buffer.Add(new PortValue(zvukOP, z0));

                return true;
            }

            return false;
        }

        public override void delta_y(ref PortValue y)
        {
            if (buffer.Count > 0)
            {
                y = buffer[0];
                buffer.RemoveAt(0);
                return;
            }

            if ( z > ez )
            {
                z *= Math.Exp(-az * TimeElapsed);
                buffer.Add(new PortValue(zvukOP, z));
            }
        }

        public override void init()
        {
            buffer.Add(new PortValue(povrsinaOP, new RavnaPovrsina(pos, a, b)));
        }

        public override double tau()
        {
            if (buffer.Count > 0) return 0;

            if ( z > ez )
            {
                return - Math.Log(1 - ez / z) / a;
            }
            return double.MaxValue;
        }
    }
}