﻿using DEVSsharp;
using Stefan_Stamenkovic_91_2014.Messages;
using Stefan_Stamenkovic_91_2014.Utils;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Stefan_Stamenkovic_91_2014.Models
{
    class Ball : Atomic
    {
        private const double G = 9.81;

        private double L;
        private double eps;
        private double initSpeed;
        private double currentSpeed;
        private double CurrentSpeed
        {
            get
            {
                return currentSpeed;
            }
            set
            {
                zSpeed = value;
                currentSpeed = value;
            }
        }
        private double zSpeed;
        private Vector position;
        private Vector direction;

        public InputPort InputPosition { get; }
        public InputPort InputDirection { get; }
        public InputPort InputSpeed { get; }
        public InputPort InputHit { get; }

        public OutputPort OutputPosition { get; }

        public double R
        {
            get
            {
                return L;
            }
        }

        public Vector Direction
        {
            get
            {
                return direction;
            }
        }

        public double Speed
        {
            get
            {
                return currentSpeed;
            }
        }
        private List<PortValue> buffer;

        public Ball(string name, TimeUnit tu, double L, double eps, double initSpeed) : base(name, tu)
        {
            this.L = L;
            this.eps = eps;
            this.initSpeed = initSpeed;

            InputPosition = AddIP("inputPosition");
            InputDirection = AddIP("inputDirection");
            InputSpeed = AddIP("inputSpeed");
            InputHit = AddIP("inputHit");

            OutputPosition = AddOP("outputPosition");
        }

        public override bool delta_x(PortValue x)
        {
            if(x.port == InputPosition)
            {
                CurrentSpeed = 0;
                position = x.value as Vector;
                buffer.Add(new PortValue(OutputPosition, position.Clone()));
            }
            else if(x.port == InputDirection)
            {
                if(position!=null && direction!=null && CurrentSpeed != 0)
                {
                    CalculatePosition();
                }

                direction = x.value as Vector;
                buffer.Add(new PortValue(OutputPosition, position.Clone()));
            }
            else if(x.port == InputSpeed)
            {
                if (position != null && direction != null)
                {
                    CalculatePosition();
                }

                CurrentSpeed = (double)x.value;
            }
            else if(x.port==InputHit)
            {
                Reflected reflected = x.value as Reflected;

                direction = reflected.NewDirection;
                currentSpeed = reflected.NewSpeed;
            }

            return true;
        }

        public override void delta_y(ref PortValue y)
        {
            if(buffer.Any())
            {
                y = buffer[0];
                buffer.RemoveAt(0);
            }
            else
            {
                CalculatePosition();
            }
        }

        public override void init()
        {
            CurrentSpeed = initSpeed;
            buffer = new List<PortValue>();
        }

        public override double tau()
        {
            if(buffer.Any())
            {
                return 0;
            }
            else
            {
                if (position != null && direction != null)
                {
                    if(CurrentSpeed==0)
                    {
                        return Double.MaxValue;
                    }
                    else
                    {
                        return CalculateTau();
                    }
                }

                return Double.MaxValue;
            }
        }

        private void CalculatePosition()
        {
            if(direction.Z>0)
            {
                zSpeed -= G * TimeElapsed;
            }
            else
            {

                zSpeed += G * TimeElapsed;
            }

            Vector m = direction.WithIntensity(currentSpeed * TimeElapsed, zSpeed * TimeElapsed);
            position = position.Clone();

            position.X += m.X;
            position.Y += m.Y;
            position.Z += m.Z;


            buffer.Add(new PortValue(OutputPosition, position.Clone()));
        }

        private double CalculateTau()
        {
            Vector m = direction.WithIntensity(currentSpeed,zSpeed);
            //
            return Math.Sqrt(eps*eps/(m.X*m.X+m.Y*m.Y+m.Z*m.Z+m.Z*G+G*G));
            //
            double a = (G * G) / 4;
            double b = -m.Z * G;
            double c = m.X * m.X + m.Y * m.Y + m.Z * m.Z - eps * eps;

            double val1 = (-b + Math.Sqrt(b * b - 4 * a * c)) / (2 * a);
            double val2 = (-b - Math.Sqrt(b * b - 4 * a * c)) / (2 * a);

            double r1 = Math.Sqrt(val1);
            double r3 = Math.Sqrt(val2);

            if(Double.IsNaN(r1))
            {
                return r3;
            }
            else if(Double.IsNaN(r3))
            {
                return r1;
            }

            return r1 < r3 ? r1 : r3;
        }

        public override string Get_s()
        {
            StringBuilder sb = new StringBuilder();

            if (position == null)
            {
                sb.Append("Position: null, ");
            }
            else
            {
                sb.Append(String.Format("Position: ({0},{1},{2}), ", position.X, position.Y, position.Z));
            }


            if (direction == null)
            {
                sb.Append("Direction: null, ");
            }
            else
            {
                sb.Append(String.Format("Direction: ({0},{1},{2}), ", direction.X, direction.Y, direction.Z));
            }

            sb.Append(String.Format("Speed: {0}", CurrentSpeed));

            return sb.ToString();
        }
    }
}
