﻿using DEVSsharp;
using Stefan_Stamenkovic_91_2014.Messages;
using Stefan_Stamenkovic_91_2014.Utils;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Stefan_Stamenkovic_91_2014.Models
{
    class Controller : Atomic
    {
        public InputPort InputMove { get; }
        public OutputPort OutputHit { get; }

        private List<FlatSurface> surfaces;
        private Ball b;

        private List<PortValue> buffer;

        public Controller(string name, TimeUnit tu, List<FlatSurface> surfaces, Ball b) : base(name, tu)
        {
            this.surfaces = surfaces;
            this.b = b;

            InputMove = AddIP("inputMove");
            OutputHit = AddOP("outputHit");
        }

        public override bool delta_x(PortValue x)
        {
            if(x.port == InputMove)
            {
                Vector v = x.value as Vector;

                foreach(FlatSurface surface in surfaces)
                {
                    if(Intersect(surface, v))
                    {
                        buffer.Add(new PortValue(OutputHit, new Intersected()
                        {
                            Id = surface.Id,
                            Direction = b.Direction,
                            Speed = b.Speed
                        }));

                        break;
                    }
                }
            }

            return true;
        }

        private bool Intersect(FlatSurface surface, Vector position)
        {
            if(surface.XY)
            {
                double up = position.Z+b.R;
                double down = position.Z - b.R;

                if(up>=surface.Center.Z && down<=surface.Center.Z && IntersectInternalXY(surface, position))
                {
                    return true;
                }
            }
            else
            {
                double left = position.X - b.R;
                double right = position.X + b.R;

                if (left<=surface.Center.X && right>= surface.Center.X && IntersectInternalYZ(surface, position))
                {
                    return true;
                }
            }

            return false;
        }

        private bool IntersectInternalXY(FlatSurface surface, Vector position)
        {
            double aHalf = surface.A / 2;
            double bHalf = surface.B / 2;

            double left = position.X - b.R;
            double right = position.X + b.R;
            double front = position.Y + b.R;
            double back = position.Y - b.R;

            return surface.Center.X + aHalf >= right && surface.Center.X - aHalf <= left
                && surface.Center.Y + bHalf >= front && surface.Center.Y - bHalf <= back;
        }

        private bool IntersectInternalYZ(FlatSurface surface, Vector position)
        {
            double aHalf = surface.A / 2;
            double bHalf = surface.B / 2;

            double up = position.Z - b.R;
            double down = position.Z + b.R;
            double front = position.Y + b.R;
            double back = position.Y - b.R;

            return surface.Center.Z + bHalf >= up && surface.Center.X - bHalf <= down
                && surface.Center.Y + aHalf >= front && surface.Center.Y - aHalf <= back;
        }

        public override void delta_y(ref PortValue y)
        {
            y = buffer[0];
            buffer.RemoveAt(0);
        }

        public override void init()
        {
            buffer = new List<PortValue>();
        }

        public override double tau()
        {
            if(buffer.Any())
            {
                return 0;
            }
            else
            {
                return Double.MaxValue;
            }
        }
    }
}
