﻿using DEVSsharp;
using Stefan_Stamenkovic_91_2014.Messages;
using Stefan_Stamenkovic_91_2014.Utils;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Stefan_Stamenkovic_91_2014.Models
{
    class FlatSurface : Atomic
    {
        private Vector center;
        private double a;
        private double b;
        private bool xy;
        
        public Vector Center
        {
            get
            {
                return center;
            }
        }

        public double A
        {
            get
            {
                return a;
            }
        }

        public double B
        {
            get
            {
                return b;
            }
        }

        public InputPort InputHit { get; }
        public OutputPort OutputHit { get; }
        public OutputPort PrintSound { get; }
        private List<PortValue> buffer;
        private double decibels = -1;
        private double kZ, epsZ;
        private Vector myVector;
        private double time;
        public string Id { get; }
        public bool XY
        {
            get
            {
                return xy;
            }
        }
        private double alpha;

        public FlatSurface(string name, TimeUnit tu, Vector center, double a, double b, bool xy, double kZ, double epsZ, double alpha) : base(name, tu)
        {
            this.center = center;
            this.a = a;
            this.b = b;
            this.kZ = kZ;
            this.epsZ = epsZ;
            this.xy = xy;
            this.alpha = alpha;
            Id = Guid.NewGuid().ToString();

            if(xy)
            {
                myVector = new Vector(0, 0, 1);
            }
            else
            {
                myVector = new Vector(1, 0, 0);
            }

            InputHit = AddIP("inputHit");
            OutputHit = AddOP("outputHit");
            PrintSound = AddOP("printSound");
        }

        public override bool delta_x(PortValue x)
        {
            if(x.port == InputHit)
            {
                Intersected intersected = x.value as Intersected;

                if(Id == intersected.Id)
                {
                    time = TimeCurrent;

                    if(xy)
                    {
                        decibels = intersected.Direction.WithIntensity(intersected.Speed).Z * kZ;
                    }
                    else
                    {
                        decibels = intersected.Direction.WithIntensity(intersected.Speed).X * kZ;
                    }

                    buffer.Add(new PortValue(OutputHit, new Reflected()
                    {
                        NewDirection = intersected.Direction.MakeSuplement(myVector),
                        NewSpeed = intersected.Speed * 0.9
                    }));
                }
            }

            return true;
        }

        public override void delta_y(ref PortValue y)
        {
            if(buffer.Any())
            {
                y = buffer[0];
                buffer.RemoveAt(0);
            }
            else
            {
                double temp = decibels * Math.Pow(Math.E, -alpha*(TimeCurrent-time));
                if(temp==0)
                {
                    decibels = -1;
                }

                buffer.Add(new PortValue(PrintSound, temp));
            }
        }

        public override void init()
        {
            buffer = new List<PortValue>();
        }

        public override double tau()
        {
            if(buffer.Any())
            {
                return 0;
            }
            else if(decibels==-1)
            {
                return Double.MaxValue;
            }
            else
            {
                double temp = decibels * Math.Pow(Math.E, -alpha * (TimeCurrent - time));

                return -Math.Log(epsZ/temp) / alpha;
            }
        }
    }
}
