﻿using DEVSsharp;
using Stefan_Stamenkovic_91_2014.Utils;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Stefan_Stamenkovic_91_2014.Models
{
    class Game : Coupled
    {
        public InputPort InputPosition { get; }
        public InputPort InputDirection { get; }
        public InputPort InputSpeed { get; }

        public Game(string name) : base(name)
        {
            Ball b = new Ball("Ball", TimeUnit.Sec, 0.2, 0.1, 5);
            FlatSurface table = new FlatSurface("Table", TimeUnit.Sec, new Vector(0, 0, 0), 10, 5, false, 0.3, 0.2, 0.25);
            FlatSurface floor = new FlatSurface("Floor", TimeUnit.Sec, new Vector(0, 0, 0), 100, 100, true, 0.3, 0.1, 0.25);

            Controller c = new Controller("Controller", TimeUnit.Sec, new List<FlatSurface>() { table,floor}, b);

            Printer p = new Printer("Printer", TimeUnit.Sec);

            AddModel(b);
            AddModel(floor);
            AddModel(table);

            AddModel(c);

            AddModel(p);

            InputPosition = AddIP("gameInputPositon");
            InputDirection = AddIP("gameInputDirection");
            InputSpeed = AddIP("gameInputSpeed");

            AddCP(InputPosition, b.InputPosition);
            AddCP(InputDirection, b.InputDirection);
            AddCP(InputSpeed, b.InputSpeed);

            AddCP(table.OutputHit, b.InputHit);
            AddCP(table.PrintSound, p.InputTable);

            AddCP(floor.OutputHit, b.InputHit);
            AddCP(floor.PrintSound, p.InputFloor);

            AddCP(c.OutputHit, table.InputHit);
            AddCP(c.OutputHit, floor.InputHit);

            AddCP(b.OutputPosition, c.InputMove);

            AddCP(b.OutputPosition, p.InputPosition);

            PrintCouplings();
        }
    }
}
