﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DEVSsharp;

namespace LazarVasovic642014
{
    class Lopta : Atomic
    {
        double rl;
        public double epsK;
        public InputPort inputNovaPozicija;
        public InputPort inputNovaBrzina;
        public OutputPort outPozicija;
        public OutputPort outKontroler;
        public Vector3D pozicija;
        public Vector3D brzina;
        public Vector3D gravitacija;
        bool stiglaBrzina;
        List<PortValue> bafer;

        public Lopta(string name, TimeUnit tu) : base(name, tu)
        {
            inputNovaPozicija = AddIP("inputNovaPozicija");
            inputNovaBrzina = AddIP("inputNovaBrzina");
            outPozicija = AddOP("outPozicija");
            outKontroler = AddOP("outKontroler");
        }

        public override bool delta_x(PortValue x)
        {
            if(x.port==inputNovaPozicija)
            {
                pozicija = (Vector3D)x.value;
                brzina = new Vector3D(0, 0, 0);
                bafer.Add(new PortValue(outPozicija, pozicija));
            }else if(x.port==inputNovaBrzina)
            {
                stiglaBrzina = true;
                pozicija = pozicija + brzina * TimeElapsed + gravitacija * (TimeElapsed * TimeElapsed / 2);
                brzina = (Vector3D)x.value;
            }

            return true;
        }

        public override void delta_y(ref PortValue y)
        {
            if(bafer.Count>0)
            {
                y = bafer[0];
                bafer.RemoveAt(0);
                return;
            }

            pozicija = pozicija + brzina * TimeElapsed + gravitacija * (TimeElapsed * TimeElapsed / 2);
            brzina = brzina + gravitacija * TimeElapsed;
            bafer.Add(new PortValue(outPozicija, pozicija));
            bafer.Add(new PortValue(outKontroler, this));
        }

        public override void init()
        {
            rl = 0.15;//u metrima
            epsK = 2 * rl;
            pozicija = new Vector3D(100, 100, 20);
            gravitacija = new Vector3D(0, 0, -9.81);
            brzina = new Vector3D(3, 3, 5);
            stiglaBrzina = true;//false;
            bafer = new List<PortValue>();
        }

        private double fX(double t)
        {
            return brzina.x * brzina.x * t * t + brzina.y * brzina.y * t * t + brzina.y * gravitacija.y * t * t * t + gravitacija.y * gravitacija.y * t * t * t * t / 4 + brzina.z * brzina.z * t * t - epsK * epsK;
        }

        private double fPrimX(double t)
        {
            return brzina.x * brzina.x * t * 2 + brzina.y * brzina.y * t * 2 + brzina.y * gravitacija.y * t * t * 3 + gravitacija.y * gravitacija.y * t * t * t + brzina.z * brzina.z * t * 2;
        }

        private double izracunajVreme()
        {
            double x0 = 0.01;
            double x1 = x0;

            while(Math.Abs(fX(x1))>1e-5)
            {
                x1 = x0 - fX(x0) / fPrimX(x0);
                x0 = x1;
            }

            return x1;
        }

        public override double tau()
        {
            if (bafer.Count > 0)
                return 0;

            if (pozicija.z == -1)
                return double.MaxValue;
            if (stiglaBrzina == false)
                return double.MaxValue;

            return izracunajVreme();

        }
    }
}
