﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using DEVSsharp;

namespace LazarVasovic642014
{
    class RavnaPovrsina : Atomic
    {
        public Vector3D centar;
        public double a;
        public double b;
        double f0;
        double alpha;
        double f;
        double kz;
        double epsZ;
        Vector3D normala;
        public string paralelnost;
        public InputPort inputUdarac;
        public OutputPort outZvuk;
        public OutputPort outOdbitakLoptice;
        public OutputPort outKontrolerTabla;
        List<PortValue> bafer;

        public RavnaPovrsina(string name, TimeUnit tu, double a, double b, Vector3D normala, Vector3D centar, string paralelnost) : base(name, tu)
        {
            this.a = a;
            this.b = b;
            this.normala = normala;
            this.paralelnost = paralelnost;
            this.centar = centar;
            inputUdarac = AddIP("inputUdarac");
            outZvuk = AddOP("outZvuk");
            outOdbitakLoptice = AddOP("outOdbitakLoptice");
            outKontrolerTabla = AddOP("outKontrolerTabla");
        }

        public override bool delta_x(PortValue x)
        {
            if(x.port==inputUdarac)
            {
                Lopta l = (Lopta)x.value;

                if(this.paralelnost=="XY")
                {
                    Vector3D novoV = l.brzina;
                    novoV.z *= -1;
                    novoV = novoV * 0.9;
                    bafer.Add(new PortValue(outOdbitakLoptice, novoV));
                }else if(this.paralelnost=="YZ")
                {
                    Vector3D novoV = l.brzina;
                    novoV.x *= -1;
                    novoV = novoV * 0.9;
                    bafer.Add(new PortValue(outOdbitakLoptice, novoV));
                }

                //zvuk
                double cos = (this.normala.DotProduct(l.brzina)) / (l.brzina.Intensity() * normala.Intensity());
                double sin = Vector3D.CrossProduct(this.normala, l.brzina).Intensity() / (l.brzina.Intensity() * normala.Intensity());
                double fi = Math.PI/2-Math.Atan2(cos, sin);

                f0 = Math.Sin(fi) * l.brzina.Intensity() * kz;
                bafer.Add(new PortValue(outZvuk, f0));

                return true;
            }

            return false;
        }

        public override void delta_y(ref PortValue y)
        {
            if(bafer.Count>0)
            {
                y = bafer[0];
                bafer.RemoveAt(0);
                return;
            }

            f = f0 * Math.Pow(Math.E, -alpha * TimeElapsed);
            bafer.Add(new PortValue(outZvuk, f));
        }

        public override void init()
        {
            bafer = new List<PortValue>();
            kz = 0.5;
            alpha = 1.2;
            epsZ = 1;
            bafer.Add(new PortValue(outKontrolerTabla,this));
        }

        public override double tau()
        {
            if (bafer.Count > 0)
            {
                return 0;
            }

            double ret = (Math.Log(f0) - Math.Log(f - epsZ)) / alpha;
            return ret;
        }
    }
}
